# -*- coding: utf-8 -*-
from functools import partial
import os
import re
from traceback import format_exc

import numpy as np
from PySide6 import QtWidgets

from ..ui import resource_rc
from ..ui.define_channel_dialog import Ui_ComputedChannel
from .advanced_search import AdvancedSearch

SIG_RE = re.compile(r"\{\{(?!\}\})(?P<name>.*?)\}\}")

SHORT_OPS = [
    "+",
    "-",
    "/",
    "//",
    "*",
    "%",
    "**",
    "^",
    "&",
    "|",
    ">>",
    "<<",
]


FUNCTIONS = [
    "absolute",
    "arccos",
    "arcsin",
    "arctan",
    "cbrt",
    "ceil",
    "clip",
    "cos",
    "cumprod",
    "cumsum",
    "deg2rad",
    "degrees",
    "diff",
    "exp",
    "fix",
    "floor",
    "gradient",
    "log",
    "log10",
    "log2",
    "rad2deg",
    "radians",
    "rint",
    "round",
    "sin",
    "sqrt",
    "square",
    "tan",
    "trunc",
]

ARGS_COUNT = {
    **{name: 0 for name in FUNCTIONS},
    "clip": 2,
    "round": 1,
}


MULTIPLE_ARGS_FUNCTIONS = {
    "clip",
    "round",
}


class DefineChannel(Ui_ComputedChannel, QtWidgets.QDialog):
    def __init__(
        self,
        mdf,
        name="",
        computation=None,
        computed_signals=None,
        origin_uuid=None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

        self.mdf = mdf
        self.result = None
        self.pressed_button = None
        self.computed_signals = computed_signals or {}
        self.origin_uuid = origin_uuid or (mdf.uuid if mdf else os.urandom(6).hex())

        self.operand1_float.setMaximum(np.inf)
        self.operand1_float.setMinimum(-np.inf)
        self.operand1_integer.setMaximum(2**31 - 1)
        self.operand1_integer.setMinimum(-(2**31) + 1)

        self.operand2_float.setMaximum(np.inf)
        self.operand2_float.setMinimum(-np.inf)
        self.operand2_integer.setMaximum(2**31 - 1)
        self.operand2_integer.setMinimum(-(2**31) + 1)

        self.first_function_argument.setMaximum(np.inf)
        self.first_function_argument.setMinimum(-np.inf)
        self.second_function_argument.setMaximum(np.inf)
        self.second_function_argument.setMinimum(-np.inf)

        for widget in (
            self.apply_btn,
            self.cancel_btn,
            self.operand1_search_btn,
            self.operand2_search_btn,
            self.function_search_btn,
            self.expression_search_btn,
        ):
            widget.setDefault(False)
            widget.setAutoDefault(False)

        self.op.addItems(
            [
                "+ (add)",
                "- (substract)",
                "/ (divide)",
                "// (floor divide)",
                "* (multiply)",
                "% (modulo)",
                "** (power)",
                "^ (xor)",
                "& (and)",
                "| (or)",
                ">> (right shift)",
                "<< (left shift)",
            ]
        )

        self.function.addItems(sorted(FUNCTIONS))
        self.function.setCurrentIndex(-1)

        self.apply_btn.clicked.connect(self.apply)
        self.cancel_btn.clicked.connect(self.cancel)

        self.operand1_search_btn.clicked.connect(
            partial(self.search, text_widget=self.operand1_name)
        )
        self.operand2_search_btn.clicked.connect(
            partial(self.search, text_widget=self.operand2_name)
        )
        self.function_search_btn.clicked.connect(
            partial(self.search, text_widget=self.function_channel)
        )
        self.expression_search_btn.clicked.connect(
            partial(self.search, text_widget=self.expression)
        )

        self.function.currentIndexChanged.connect(self.function_changed)

        if computation is None:

            if name:
                self.operand1_name.setText(name)
                self.function_channel.setText(name)

        else:

            self.name.setText(
                computation.get("channel_name", computation.get("channel", ""))
            )
            self.unit.setText(computation.get("channel_unit", ""))
            self.comment.setPlainText(computation.get("channel_comment", ""))

            if computation["type"] == "arithmetic":
                if isinstance(computation["operand1"], str):
                    self.operand1_as_signal.setChecked(True)
                    if not isinstance(computation["operand1"], dict):
                        self.operand1_name.setText(computation["operand1"])
                elif isinstance(computation["operand1"], int):
                    self.operand1_as_integer.setChecked(True)
                    self.operand1_integer.setValue(computation["operand1"])
                else:
                    self.operand1_as_float.setChecked(True)
                    self.operand1_float.setValue(computation["operand1"])

                if isinstance(computation["operand2"], str):
                    self.operand2_as_signal.setChecked(True)
                    if not isinstance(computation["operand2"], dict):
                        self.operand2_name.setText(computation["operand2"])
                elif isinstance(computation["operand2"], int):
                    self.operand2_as_integer.setChecked(True)
                    self.operand2_integer.setValue(computation["operand2"])
                else:
                    self.operand2_as_float.setChecked(True)
                    self.operand2_float.setValue(computation["operand2"])

                self.op.setCurrentIndex(SHORT_OPS.index(computation["op"]))

            elif computation["type"] == "function":
                self.tabs.setCurrentIndex(1)
                self.function.setCurrentText(computation["name"])
                if not isinstance(computation["channel"], dict):
                    self.function_channel.setText(computation["channel"])

                for arg, arg_value in zip(
                    [self.first_function_argument, self.second_function_argument],
                    computation["args"],
                ):
                    arg.setValue(arg_value)

            else:
                self.tabs.setCurrentIndex(2)
                self.expression.setPlainText(computation["expression"])

    def apply_simple_computation(self):

        if (
            self.operand1_as_signal.isChecked()
            and not self.operand1_name.text().strip()
        ):
            QtWidgets.QMessageBox.warning(
                None, "Can't compute new channel", "Must select operand 1 first"
            )
            return

        if (
            self.operand2_as_signal.isChecked()
            and not self.operand2_name.text().strip()
        ):
            QtWidgets.QMessageBox.warning(
                None, "Can't compute new channel", "Must select operand 2 first"
            )
            return

        if self.op.currentIndex() == -1:
            QtWidgets.QMessageBox.warning(
                None, "Can't compute new channel", "Must select operator"
            )
            return

        if self.operand1_as_signal.isChecked():
            operand1 = self.operand1_name.text().strip()
            if operand1 in self.computed_signals:
                operand1 = self.computed_signals[operand1].computation
        elif self.operand1_as_integer.isChecked():
            operand1 = self.opernad1_integer.value()
        else:
            operand1 = self.operand1_float.value()

        if self.operand2_as_signal.isChecked():
            operand2 = self.operand2_name.text().strip()
            if operand2 in self.computed_signals:
                operand2 = self.computed_signals[operand2].computation
        elif self.operand2_as_integer.isChecked():
            operand2 = self.operand2_integer.value()
        else:
            operand2 = self.operand2_float.value()

        op = self.op.currentText().split(" ")[0]

        self.result = {
            "type": "channel",
            "common_axis": False,
            "individual_axis": False,
            "enabled": True,
            "mode": "phys",
            "fmt": "{:.3f}",
            "format": "phys",
            "precision": 3,
            "ranges": [],
            "y_range": [0, 1],
            "unit": self.unit.text().strip(),
            "computed": True,
            "color": f"#{os.urandom(3).hex()}",
            "origin_uuid": self.origin_uuid,
            "uuid": os.urandom(6).hex(),
            "group_index": -1,
            "channel_index": -1,
            "name": self.name.text().strip() or f"{operand1}_{op}_{operand2}",
            "computation": {
                "type": "arithmetic",
                "op": op,
                "operand1": operand1,
                "operand2": operand2,
                "channel_name": self.name.text().strip()
                or f"{operand1}_{op}_{operand2}",
                "channel_unit": self.unit.text().strip(),
                "channel_comment": self.comment.toPlainText().strip(),
            },
        }

        self.pressed_button = "apply"
        self.close()

    def function_changed(self, index):
        function = self.function.currentText()
        self.help.setText(getattr(np, function).__doc__)

        args = ARGS_COUNT[function]
        if args == 0:
            self.first_function_argument.setEnabled(False)
            self.second_function_argument.setEnabled(False)
        elif args == 1:
            self.first_function_argument.setEnabled(True)
            self.second_function_argument.setEnabled(False)
        else:
            self.first_function_argument.setEnabled(True)
            self.second_function_argument.setEnabled(True)

    def apply_function(self):
        if self.function.currentIndex() == -1:
            QtWidgets.QMessageBox.warning(
                None, "Can't compute new channel", "Must select a function first"
            )
            return

        if not self.function_channel.text().strip():
            QtWidgets.QMessageBox.warning(
                None, "Can't compute new channel", "Must select a channel first"
            )
            return

        function = self.function.currentText()
        function_channel = self.function_channel.text().strip()

        args_count = ARGS_COUNT[function]
        if args_count == 0:
            args = []
        elif args_count == 1:
            args = [self.first_function_argument.value()]
        else:
            args = [
                self.first_function_argument.value(),
                self.second_function_argument.value(),
            ]

        if function_channel in self.computed_signals:
            function_channel = self.computed_signals[function_channel].computation

        self.result = {
            "type": "channel",
            "common_axis": False,
            "individual_axis": False,
            "enabled": True,
            "mode": "phys",
            "fmt": "{:.3f}",
            "format": "phys",
            "precision": 3,
            "ranges": [],
            "y_range": [0, 1],
            "unit": self.unit.text().strip(),
            "computed": True,
            "color": f"#{os.urandom(3).hex()}",
            "uuid": os.urandom(6).hex(),
            "origin_uuid": self.origin_uuid,
            "group_index": -1,
            "channel_index": -1,
            "name": self.name.text().strip() or f"{function}({function_channel})",
            "computation": {
                "type": "function",
                "channel": function_channel,
                "name": function,
                "args": args,
                "channel_name": self.name.text().strip()
                or f"{function}({function_channel})",
                "channel_unit": self.unit.text().strip(),
                "channel_comment": self.comment.toPlainText().strip(),
            },
        }

        self.pressed_button = "apply"
        self.close()

    def apply(self, event):
        if self.tabs.currentIndex() == 0:
            self.apply_simple_computation()
        elif self.tabs.currentIndex() == 1:
            self.apply_function()
        else:
            self.apply_expression()
        self.close()

    def apply_expression(self):
        expression_string = self.expression.toPlainText().strip()
        expression_string = "".join(expression_string.splitlines())

        self.result = {
            "type": "channel",
            "common_axis": False,
            "individual_axis": False,
            "enabled": True,
            "mode": "phys",
            "fmt": "{:.3f}",
            "format": "phys",
            "precision": 3,
            "ranges": [],
            "y_range": [0, 1],
            "unit": self.unit.text().strip(),
            "computed": True,
            "color": f"#{os.urandom(3).hex()}",
            "uuid": os.urandom(6).hex(),
            "origin_uuid": self.origin_uuid,
            "group_index": -1,
            "channel_index": -1,
            "name": self.name.text().strip() or "expression",
            "computation": {
                "type": "expression",
                "expression": expression_string,
                "channel_name": self.name.text().strip() or "expression",
                "channel_unit": self.unit.text().strip(),
                "channel_comment": self.comment.toPlainText().strip(),
            },
        }

        self.pressed_button = "apply"
        self.close()

    def cancel(self, event):
        self.result = None
        self.pressed_button = "cancel"
        self.close()

    def search(self, *args, text_widget=None):
        dlg = AdvancedSearch(
            self.mdf,
            show_add_window=False,
            show_apply=True,
            apply_text="Select channel",
            show_pattern=False,
            parent=self,
            return_names=True,
            computed_signals=self.computed_signals,
        )
        dlg.setModal(True)
        dlg.exec_()
        result, pattern_window = dlg.result, dlg.pattern_window

        if result:
            text = list(result)[0]

            if text_widget is self.expression:
                text_widget.insertPlainText("{{" + text + "}}")
            elif text_widget is not None:
                text_widget.setText(text)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'define_channel_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDialog, QDoubleSpinBox,
    QGridLayout, QGroupBox, QLabel, QLineEdit,
    QPlainTextEdit, QPushButton, QRadioButton, QSizePolicy,
    QSpacerItem, QSpinBox, QTabWidget, QTextEdit,
    QWidget)
from . import resource_rc

class Ui_ComputedChannel(object):
    def setupUi(self, ComputedChannel):
        if not ComputedChannel.objectName():
            ComputedChannel.setObjectName(u"ComputedChannel")
        ComputedChannel.resize(937, 480)
        ComputedChannel.setMaximumSize(QSize(16777215, 16777215))
        icon = QIcon()
        icon.addFile(u":/plus.png", QSize(), QIcon.Normal, QIcon.Off)
        ComputedChannel.setWindowIcon(icon)
        ComputedChannel.setSizeGripEnabled(True)
        self.gridLayout = QGridLayout(ComputedChannel)
        self.gridLayout.setObjectName(u"gridLayout")
        self.tabs = QTabWidget(ComputedChannel)
        self.tabs.setObjectName(u"tabs")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.gridLayout_5 = QGridLayout(self.tab)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.groupBox = QGroupBox(self.tab)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout_4 = QGridLayout(self.groupBox)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.operand1_as_signal = QRadioButton(self.groupBox)
        self.operand1_as_signal.setObjectName(u"operand1_as_signal")
        self.operand1_as_signal.setChecked(True)

        self.gridLayout_4.addWidget(self.operand1_as_signal, 0, 0, 1, 1)

        self.operand1_name = QLineEdit(self.groupBox)
        self.operand1_name.setObjectName(u"operand1_name")

        self.gridLayout_4.addWidget(self.operand1_name, 0, 1, 1, 2)

        self.operand1_search_btn = QPushButton(self.groupBox)
        self.operand1_search_btn.setObjectName(u"operand1_search_btn")
        icon1 = QIcon()
        icon1.addFile(u":/search.png", QSize(), QIcon.Normal, QIcon.Off)
        self.operand1_search_btn.setIcon(icon1)

        self.gridLayout_4.addWidget(self.operand1_search_btn, 0, 3, 1, 1)

        self.operand1_as_integer = QRadioButton(self.groupBox)
        self.operand1_as_integer.setObjectName(u"operand1_as_integer")

        self.gridLayout_4.addWidget(self.operand1_as_integer, 1, 0, 1, 1)

        self.operand1_integer = QSpinBox(self.groupBox)
        self.operand1_integer.setObjectName(u"operand1_integer")
        self.operand1_integer.setMinimumSize(QSize(200, 0))
        self.operand1_integer.setMinimum(0)
        self.operand1_integer.setMaximum(10)
        self.operand1_integer.setValue(0)

        self.gridLayout_4.addWidget(self.operand1_integer, 1, 1, 1, 1)

        self.horizontalSpacer = QSpacerItem(538, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_4.addItem(self.horizontalSpacer, 1, 2, 2, 1)

        self.operand1_as_float = QRadioButton(self.groupBox)
        self.operand1_as_float.setObjectName(u"operand1_as_float")

        self.gridLayout_4.addWidget(self.operand1_as_float, 2, 0, 1, 1)

        self.operand1_float = QDoubleSpinBox(self.groupBox)
        self.operand1_float.setObjectName(u"operand1_float")
        self.operand1_float.setDecimals(6)

        self.gridLayout_4.addWidget(self.operand1_float, 2, 1, 1, 1)

        self.gridLayout_4.setColumnStretch(2, 1)

        self.gridLayout_5.addWidget(self.groupBox, 0, 0, 1, 2)

        self.op = QComboBox(self.tab)
        self.op.setObjectName(u"op")

        self.gridLayout_5.addWidget(self.op, 1, 0, 1, 1)

        self.horizontalSpacer_5 = QSpacerItem(794, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_5.addItem(self.horizontalSpacer_5, 1, 1, 1, 1)

        self.groupBox_2 = QGroupBox(self.tab)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_3 = QGridLayout(self.groupBox_2)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.operand2_as_signal = QRadioButton(self.groupBox_2)
        self.operand2_as_signal.setObjectName(u"operand2_as_signal")
        self.operand2_as_signal.setChecked(True)

        self.gridLayout_3.addWidget(self.operand2_as_signal, 0, 0, 1, 1)

        self.operand2_name = QLineEdit(self.groupBox_2)
        self.operand2_name.setObjectName(u"operand2_name")

        self.gridLayout_3.addWidget(self.operand2_name, 0, 1, 1, 2)

        self.operand2_search_btn = QPushButton(self.groupBox_2)
        self.operand2_search_btn.setObjectName(u"operand2_search_btn")
        self.operand2_search_btn.setIcon(icon1)

        self.gridLayout_3.addWidget(self.operand2_search_btn, 0, 3, 1, 1)

        self.operand2_as_integer = QRadioButton(self.groupBox_2)
        self.operand2_as_integer.setObjectName(u"operand2_as_integer")

        self.gridLayout_3.addWidget(self.operand2_as_integer, 1, 0, 1, 1)

        self.operand2_integer = QSpinBox(self.groupBox_2)
        self.operand2_integer.setObjectName(u"operand2_integer")
        self.operand2_integer.setMinimumSize(QSize(200, 0))

        self.gridLayout_3.addWidget(self.operand2_integer, 1, 1, 1, 1)

        self.horizontalSpacer_2 = QSpacerItem(63, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_3.addItem(self.horizontalSpacer_2, 1, 2, 1, 1)

        self.operand2_as_float = QRadioButton(self.groupBox_2)
        self.operand2_as_float.setObjectName(u"operand2_as_float")

        self.gridLayout_3.addWidget(self.operand2_as_float, 2, 0, 1, 1)

        self.operand2_float = QDoubleSpinBox(self.groupBox_2)
        self.operand2_float.setObjectName(u"operand2_float")
        self.operand2_float.setDecimals(6)

        self.gridLayout_3.addWidget(self.operand2_float, 2, 1, 1, 1)

        self.gridLayout_3.setColumnStretch(2, 1)

        self.gridLayout_5.addWidget(self.groupBox_2, 2, 0, 1, 2)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_5.addItem(self.verticalSpacer, 5, 0, 1, 1)

        self.gridLayout_5.setColumnStretch(1, 1)
        self.tabs.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName(u"tab_2")
        self.gridLayout_6 = QGridLayout(self.tab_2)
        self.gridLayout_6.setObjectName(u"gridLayout_6")
        self.label_2 = QLabel(self.tab_2)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_6.addWidget(self.label_2, 0, 0, 1, 1)

        self.label_3 = QLabel(self.tab_2)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout_6.addWidget(self.label_3, 1, 0, 1, 1)

        self.first_function_argument = QDoubleSpinBox(self.tab_2)
        self.first_function_argument.setObjectName(u"first_function_argument")

        self.gridLayout_6.addWidget(self.first_function_argument, 1, 1, 1, 1)

        self.horizontalSpacer_4 = QSpacerItem(286, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_6.addItem(self.horizontalSpacer_4, 1, 2, 1, 1)

        self.label_4 = QLabel(self.tab_2)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_6.addWidget(self.label_4, 2, 0, 1, 1)

        self.second_function_argument = QDoubleSpinBox(self.tab_2)
        self.second_function_argument.setObjectName(u"second_function_argument")

        self.gridLayout_6.addWidget(self.second_function_argument, 2, 1, 1, 1)

        self.label_5 = QLabel(self.tab_2)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_6.addWidget(self.label_5, 3, 0, 1, 1)

        self.function_channel = QLineEdit(self.tab_2)
        self.function_channel.setObjectName(u"function_channel")

        self.gridLayout_6.addWidget(self.function_channel, 3, 1, 1, 2)

        self.function_search_btn = QPushButton(self.tab_2)
        self.function_search_btn.setObjectName(u"function_search_btn")
        self.function_search_btn.setIcon(icon1)

        self.gridLayout_6.addWidget(self.function_search_btn, 3, 3, 1, 1)

        self.function = QComboBox(self.tab_2)
        self.function.setObjectName(u"function")
        self.function.setMinimumSize(QSize(380, 20))

        self.gridLayout_6.addWidget(self.function, 0, 1, 1, 1)

        self.help = QTextEdit(self.tab_2)
        self.help.setObjectName(u"help")

        self.gridLayout_6.addWidget(self.help, 4, 0, 1, 4)

        self.gridLayout_6.setColumnStretch(2, 1)
        self.tabs.addTab(self.tab_2, "")
        self.tab_3 = QWidget()
        self.tab_3.setObjectName(u"tab_3")
        self.gridLayout_2 = QGridLayout(self.tab_3)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.label = QLabel(self.tab_3)
        self.label.setObjectName(u"label")
        self.label.setTextFormat(Qt.RichText)

        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 2)

        self.expression = QPlainTextEdit(self.tab_3)
        self.expression.setObjectName(u"expression")

        self.gridLayout_2.addWidget(self.expression, 2, 0, 1, 2)

        self.expression_search_btn = QPushButton(self.tab_3)
        self.expression_search_btn.setObjectName(u"expression_search_btn")
        self.expression_search_btn.setIcon(icon1)

        self.gridLayout_2.addWidget(self.expression_search_btn, 1, 0, 1, 1)

        self.gridLayout_2.setColumnStretch(1, 1)
        self.tabs.addTab(self.tab_3, "")

        self.gridLayout.addWidget(self.tabs, 0, 0, 1, 5)

        self.cancel_btn = QPushButton(ComputedChannel)
        self.cancel_btn.setObjectName(u"cancel_btn")

        self.gridLayout.addWidget(self.cancel_btn, 5, 4, 1, 1)

        self.label_8 = QLabel(ComputedChannel)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)

        self.gridLayout.addWidget(self.label_8, 3, 0, 1, 1)

        self.comment = QPlainTextEdit(ComputedChannel)
        self.comment.setObjectName(u"comment")

        self.gridLayout.addWidget(self.comment, 3, 1, 1, 4)

        self.apply_btn = QPushButton(ComputedChannel)
        self.apply_btn.setObjectName(u"apply_btn")

        self.gridLayout.addWidget(self.apply_btn, 5, 3, 1, 1)

        self.label_7 = QLabel(ComputedChannel)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout.addWidget(self.label_7, 1, 0, 1, 1)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_7, 5, 2, 1, 1)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_3, 5, 1, 1, 1)

        self.name = QLineEdit(ComputedChannel)
        self.name.setObjectName(u"name")

        self.gridLayout.addWidget(self.name, 1, 1, 1, 4)

        self.label_6 = QLabel(ComputedChannel)
        self.label_6.setObjectName(u"label_6")

        self.gridLayout.addWidget(self.label_6, 4, 0, 1, 1)

        self.unit = QLineEdit(ComputedChannel)
        self.unit.setObjectName(u"unit")

        self.gridLayout.addWidget(self.unit, 4, 1, 1, 1)

        self.gridLayout.setColumnStretch(2, 1)

        self.retranslateUi(ComputedChannel)

        self.tabs.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(ComputedChannel)
    # setupUi

    def retranslateUi(self, ComputedChannel):
        ComputedChannel.setWindowTitle(QCoreApplication.translate("ComputedChannel", u"Define new channel", None))
        self.groupBox.setTitle(QCoreApplication.translate("ComputedChannel", u"First operand", None))
        self.operand1_as_signal.setText(QCoreApplication.translate("ComputedChannel", u"Signal", None))
        self.operand1_name.setPlaceholderText(QCoreApplication.translate("ComputedChannel", u"first operand name", None))
        self.operand1_search_btn.setText("")
        self.operand1_as_integer.setText(QCoreApplication.translate("ComputedChannel", u"Integer", None))
        self.operand1_as_float.setText(QCoreApplication.translate("ComputedChannel", u"Float", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("ComputedChannel", u"Second operand", None))
        self.operand2_as_signal.setText(QCoreApplication.translate("ComputedChannel", u"Signal", None))
        self.operand2_name.setPlaceholderText(QCoreApplication.translate("ComputedChannel", u"second operand name", None))
        self.operand2_search_btn.setText("")
        self.operand2_as_integer.setText(QCoreApplication.translate("ComputedChannel", u"Integer", None))
        self.operand2_as_float.setText(QCoreApplication.translate("ComputedChannel", u"Float", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tab), QCoreApplication.translate("ComputedChannel", u"Simple computation", None))
        self.label_2.setText(QCoreApplication.translate("ComputedChannel", u"Function", None))
        self.label_3.setText(QCoreApplication.translate("ComputedChannel", u"First function argument", None))
        self.label_4.setText(QCoreApplication.translate("ComputedChannel", u"Second function argument", None))
        self.label_5.setText(QCoreApplication.translate("ComputedChannel", u"Channel name", None))
        self.function_channel.setPlaceholderText(QCoreApplication.translate("ComputedChannel", u"input channel name", None))
        self.function_search_btn.setText("")
#if QT_CONFIG(tooltip)
        self.function.setToolTip(QCoreApplication.translate("ComputedChannel", u"see numpy documentation", None))
#endif // QT_CONFIG(tooltip)
        self.help.setPlaceholderText(QCoreApplication.translate("ComputedChannel", u"selected function help ", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_2), QCoreApplication.translate("ComputedChannel", u"Function", None))
        self.label.setText(QCoreApplication.translate("ComputedChannel", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The expression is evaluated using the <span style=\" font-style:italic;\">numexpr </span>library. Have a look at the <a href=\"https://numexpr.readthedocs.io/projects/NumExpr3/en/latest/user_guide.html\"><span style=\" text-decoration: underline; color:#0000ff;\">Numexpr documentation</span></a> for supported operators and function.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:"
                        "0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The signal names must be enclosed by double curly braces: <span style=\" font-weight:600; color:#ff0000;\">{{</span>Signal_name<span style=\" font-weight:600; color:#ff0000;\">}}</span></p></body></html>", None))
        self.expression.setPlaceholderText(QCoreApplication.translate("ComputedChannel", u"enter your expression here. Example: ({{Sig1}} + {{Sig2}}) / ({{Sig3}} - {{Sig4}}) + 7.8", None))
        self.expression_search_btn.setText(QCoreApplication.translate("ComputedChannel", u"Available channels", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_3), QCoreApplication.translate("ComputedChannel", u"Expression", None))
        self.cancel_btn.setText(QCoreApplication.translate("ComputedChannel", u"Cancel", None))
        self.label_8.setText(QCoreApplication.translate("ComputedChannel", u"Computed channel comment", None))
        self.apply_btn.setText(QCoreApplication.translate("ComputedChannel", u"Apply", None))
        self.label_7.setText(QCoreApplication.translate("ComputedChannel", u"Computed channel name", None))
        self.name.setInputMask("")
        self.name.setText("")
        self.name.setPlaceholderText("")
        self.label_6.setText(QCoreApplication.translate("ComputedChannel", u"Computed channel unit", None))
        self.unit.setPlaceholderText("")
    # retranslateUi

